@@ -52,6 +52,9 @@ enum format_opt
 	FMT_NAME,
 	FMT_TYPE,
 	FMT_ADDR,
+	FMT_VOTES,
+	FMT_EXP,
+	FMT_STATE,
 };
 
 struct commandline
@@ -86,7 +89,6 @@ struct commandline
 	int clustername_opt;
 	int wait_opt;
 	int wait_quorate_opt;
-	int fence_opt;
 	int addresses_opt;
 	int noconfig_opt;
 	int nosetpri_opt;
@@ -95,6 +97,6 @@ struct commandline
 typedef struct commandline commandline_t;
 
 int join(commandline_t *comline, char *envp[]);
-char *cman_error(int err);
+const char *cman_error(int err);
 
 #endif  /*  __CMAN_TOOL_DOT_H__  */
