@@ -10,7 +10,7 @@
 static char *argv[MAX_ARGS];
 static char *envp[MAX_ARGS];
 
-static void be_daemon(int close_stderr)
+static void be_daemon(void)
 {
 	int devnull = open("/dev/null", O_RDWR);
 	if (devnull == -1) {
@@ -27,15 +27,10 @@ static void be_daemon(int close_stderr)
 		die("Error setting terminal FDs to /dev/null: %m");
 	}
 
-	if (close_stderr) {
-		if (close(2)) {
-			die("Error closing stderr FD");
-		}
-		if (!dup2(devnull, 2) < 0) {
-			die("Error setting stderr FD to /dev/null: %m");
-		}
-	}
-
+	/* We leave stderr open to allow error messags through.
+	   the cman plugin will close it when it's all started
+	   up properly.
+	*/
 	setsid();
 }
 
@@ -116,7 +111,7 @@ int join(commandline_t *comline, char *main_envp[])
 	int envptr = 0;
 	int argvptr = 0;
 	char scratch[1024];
-	cman_handle_t h;
+	cman_handle_t h = NULL;
 	int status;
 	pid_t corosync_pid;
 	int p[2];
@@ -226,7 +221,7 @@ int join(commandline_t *comline, char *main_envp[])
 				fprintf(stderr, "%s\n", envp[i]);
 			}
 		}
-		be_daemon(!(comline->verbose & ~DEBUG_STARTUP_ONLY));
+		be_daemon();
 
 		sprintf(scratch, "FORKED: %d\n", getpid());
 		err = write(p[1], scratch, strlen(scratch));
@@ -270,11 +265,11 @@ int join(commandline_t *comline, char *main_envp[])
 				if (sscanf(messageptr, "FORKED: %d", &corosync_pid) == 1) {
 					if (comline->verbose & DEBUG_STARTUP_ONLY)
 						fprintf(stderr, "forked process ID is %d\n", corosync_pid);
-					status = 1;
+					status = 0;
 
-					/* There might be a SUCCESS message in the pipe too. */
+					/* There might be a SUCCESS or error message in the pipe too. */
 					messageptr = strchr(messageptr, '\n');
-					if (messageptr)
+					if (messageptr && strlen(messageptr) > 1)
 						messageptr++;
 					else
 						continue;
@@ -283,9 +278,13 @@ int join(commandline_t *comline, char *main_envp[])
 				if (sscanf(messageptr, "SUCCESS: %d", &corosync_pid) == 1) {
 					if (comline->verbose & DEBUG_STARTUP_ONLY)
 						fprintf(stderr, "corosync running, process ID is %d\n", corosync_pid);
-					status = 0;
 					break;
 				}
+				else if (messageptr) {
+						fprintf(stderr, "%s\n", messageptr);
+						status = 1;
+						break;
+					}
 			}
 			else if (len < 0 && errno == EINTR) {
 				continue;
@@ -295,7 +294,7 @@ int join(commandline_t *comline, char *main_envp[])
 			}
 		}
 
-	} while (status != 0);
+	} while (status == 0);
 	close(p[0]);
 
 	/* If corosync has started, try to connect to cman ... if it's still there */
