@@ -222,7 +222,7 @@ static int loopy_writev(int fd, struct iovec *iovptr, size_t iovlen)
 		if ((ssize_t)iovlen <=0 )
 			break;
 
-		iovptr->iov_base += len;
+		iovptr->iov_base = (char *)iovptr->iov_base + len;
 		iovptr->iov_len -= len;
 	}
 	return byte_cnt;
@@ -346,7 +346,7 @@ int cman_finish(cman_handle_t handle)
 	return 0;
 }
 
-int cman_setprivdata(cman_handle_t *handle, void *privdata)
+int cman_setprivdata(cman_handle_t handle, void *privdata)
 {
 	struct cman_handle *h = (struct cman_handle *)handle;
 	VALIDATE_HANDLE(h);
@@ -355,7 +355,7 @@ int cman_setprivdata(cman_handle_t *handle, void *privdata)
 	return 0;
 }
 
-int cman_getprivdata(cman_handle_t *handle, void **privdata)
+int cman_getprivdata(cman_handle_t handle, void **privdata)
 {
 	struct cman_handle *h = (struct cman_handle *)handle;
 	VALIDATE_HANDLE(h);
@@ -464,6 +464,7 @@ int cman_dispatch(cman_handle_t handle, int flags)
 			res = process_cman_message(h, flags, smsg->msg);
 			h->saved_reply_msg = smsg->next;
 			len = smsg->msg->length;
+			free(smsg->msg);
 			free(smsg);
 			if (res || (flags & CMAN_DISPATCH_ONE))
 				break;
@@ -477,6 +478,7 @@ int cman_dispatch(cman_handle_t handle, int flags)
 			res = process_cman_message(h, flags, smsg->msg);
 			h->saved_data_msg = smsg->next;
 			len = smsg->msg->length;
+			free(smsg->msg);
 			free(smsg);
 			if (res || (flags & CMAN_DISPATCH_ONE))
 				break;
@@ -490,6 +492,7 @@ int cman_dispatch(cman_handle_t handle, int flags)
 			res = process_cman_message(h, flags, smsg->msg);
 			h->saved_event_msg = smsg->next;
 			len = smsg->msg->length;
+			free(smsg->msg);
 			free(smsg);
 			if (res || (flags & CMAN_DISPATCH_ONE))
 				break;
@@ -684,6 +687,20 @@ int cman_get_node(cman_handle_t handle, int nodeid, cman_node_t *node)
 	return 0;
 }
 
+int cman_get_node_extra(cman_handle_t handle, int nodeid, cman_node_extra_t *node)
+{
+	struct cman_handle *h = (struct cman_handle *)handle;
+	int status;
+	VALIDATE_HANDLE(h);
+
+	status = info_call(h, CMAN_CMD_GETNODE_EXTRA, &nodeid, sizeof(int),
+			   node, sizeof(cman_node_extra_t));
+	if (status < 0)
+		return -1;
+
+	return 0;
+}
+
 int cman_get_subsys_count(cman_handle_t handle)
 {
 	struct cman_handle *h = (struct cman_handle *)handle;
@@ -1093,13 +1110,3 @@ int cman_node_fenced(cman_handle_t handle, int nodeid, uint64_t time, char *agen
 	strcpy(f.fence_agent, agent);
 	return info_call(h, CMAN_CMD_UPDATE_FENCE_INFO, &f, sizeof(f), NULL, 0);
 }
-
-#ifdef DEBUG
-int cman_dump_objdb(cman_handle_t handle, char *filename)
-{
-	struct cman_handle *h = (struct cman_handle *)handle;
-	VALIDATE_HANDLE(h);
-
-	return info_call(h, CMAN_CMD_DUMP_OBJDB, filename, strlen(filename)+1, NULL, 0);
-}
-#endif
