@@ -11,13 +11,11 @@
 
 #include <libxml/tree.h>
 
-#ifdef LEGACY_CODE
-#include "update.h"
-#endif
+#include "editconf.h"
 
 #define MAX_NODES 256
 
-char *prog_name = "ccs_tool";
+const char *prog_name = "ccs_tool";
 
 #define die(fmt, args...) \
 do { \
@@ -29,17 +27,15 @@ do { \
 
 struct option_info
 {
-	char *name;
-	char *altname;
-	char *votes;
-	char *nodeid;
-	char *mcast_addr;
-	char *fence_type;
-	char *configfile;
-	char *outputfile;
+	const char *name;
+	const char *altname;
+	const char *votes;
+	const char *nodeid;
+	const char *mcast_addr;
+	const char *fence_type;
+	const char *configfile;
+	const char *outputfile;
 	int  do_delete;
-	int  tell_ccsd;
-	int  force_ccsd;
 };
 
 static void config_usage(int rw)
@@ -48,9 +44,6 @@ static void config_usage(int rw)
 	if (rw)
 	{
 		fprintf(stderr, " -o --outputfile    Name of output file (defaults to same as --configfile)\n");
-		fprintf(stderr, " -C --no_ccs        Don't tell CCSD about this change\n");
-		fprintf(stderr, "                    default: run \"ccs_tool update\" if file is updated in place)\n");
-		fprintf(stderr, " -F --force_ccs     Force \"ccs_tool upgrade\" even if input & output files differ\n");
 	}
 }
 
@@ -73,12 +66,14 @@ static void create_usage(const char *name)
 {
 	fprintf(stderr, "Usage: %s %s [-2] <clustername>\n", prog_name, name);
 	fprintf(stderr, " -2                 Create a 2-node cman cluster config file\n");
+	fprintf(stderr, " -n <num>           Create skeleton entries for <num> nodes\n");
+	fprintf(stderr, " -f <device>        Add a fence device to the node skeletons\n");
 	config_usage(0);
 	help_usage();
 	fprintf(stderr, "\n"
 	  "Note that \"create\" on its own will not create a valid configuration file.\n"
 	  "Fence agents and nodes will need to be added to it before handing it over\n"
-	  "to ccsd.\n"
+	  "to cman.\n"
 	  "\n"
 	  "eg:\n"
 	  "  ccs_tool create MyCluster\n"
@@ -88,6 +83,9 @@ static void create_usage(const char *name)
 	  "  ccs_tool addnode node3 -n 3 -f apc port=3\n"
 	  "  ccs_tool addnode node4 -n 4 -f apc port=4\n"
           "\n");
+	fprintf(stderr, "If you add -n <numbner> to the command then %s will add skeleton entries for\n", name);
+	fprintf(stderr, "that many nodes. This file WILL NEED EDITTING MANUALLY before it can be used\n");
+	fprintf(stderr, "by cman.\n");
 
 	exit(0);
 }
@@ -154,10 +152,10 @@ static void addnode_usage(const char *name)
 	fprintf(stderr, "Examples:\n");
 	fprintf(stderr, "\n");
 	fprintf(stderr, "Add a new node to default configuration file:\n");
-	fprintf(stderr, "  %s %s -n 1 -f manual ipaddr=newnode\n", prog_name, name);
+	fprintf(stderr, "  %s %s newnode1 -n 1 -f manual ipaddr=newnode\n", prog_name, name);
 	fprintf(stderr, "\n");
 	fprintf(stderr, "Add a new node and dump config file to stdout rather than save it\n");
-	fprintf(stderr, "  %s %s -n 2 -f apc -o- newnode.temp.net port=1\n", prog_name, name);
+	fprintf(stderr, "  %s %s newnode2 -n 2 -f apc -o- newnode.temp.net port=1\n", prog_name, name);
 
 	exit(0);
 }
@@ -184,8 +182,8 @@ static int valid_mcast_addr(char *mcast)
 
 static void save_file(xmlDoc *doc, struct option_info *ninfo)
 {
-	char tmpfile[strlen(ninfo->outputfile)+5];
-	char oldfile[strlen(ninfo->outputfile)+5];
+	char tmpconffile[strlen(ninfo->outputfile)+5];
+	char oldconffile[strlen(ninfo->outputfile)+5];
 	int using_stdout = 0;
 	mode_t old_mode;
 	int ret;
@@ -200,44 +198,34 @@ static void save_file(xmlDoc *doc, struct option_info *ninfo)
 	 */
 	if (!using_stdout)
 	{
-		snprintf(tmpfile, sizeof(tmpfile), "%s.tmp", ninfo->outputfile);
-		snprintf(oldfile, sizeof(oldfile), "%s.old", ninfo->outputfile);
+		snprintf(tmpconffile, sizeof(tmpconffile), "%s.tmp", ninfo->outputfile);
+		snprintf(oldconffile, sizeof(oldconffile), "%s.old", ninfo->outputfile);
 	}
 	else
 	{
-		strcpy(tmpfile, ninfo->outputfile);
+		strcpy(tmpconffile, ninfo->outputfile);
 	}
 
 	xmlKeepBlanksDefault(0);
-	ret = xmlSaveFormatFile(tmpfile, doc, 1);
+	ret = xmlSaveFormatFile(tmpconffile, doc, 1);
 	if (ret == -1)
 		die("Error writing new config file %s", ninfo->outputfile);
 
 	if (!using_stdout)
 	{
-		if (rename(ninfo->outputfile, oldfile) == -1 && errno != ENOENT)
+		if (rename(ninfo->outputfile, oldconffile) == -1 && errno != ENOENT)
 			die("Can't move old config file out of the way\n");
 
-		if (rename(tmpfile, ninfo->outputfile))
+		if (rename(tmpconffile, ninfo->outputfile))
 		{
 			perror("Error renaming new file to its real filename");
 
 			/* Drat, that failed, try to put the old one back */
-			if (rename(oldfile, ninfo->outputfile))
+			if (rename(oldconffile, ninfo->outputfile))
 				die("Can't move old config fileback in place - clean up after me please\n");
 		}
 	}
 
-#ifdef LEGACY_CODE
-	/* Try to tell ccsd if needed */
-	if ((strcmp(ninfo->configfile, ninfo->outputfile) == 0 && ninfo->tell_ccsd) ||
-	    ninfo->force_ccsd)
-	{
-		printf("running ccs_tool update...\n");
-		update(ninfo->outputfile);
-	}
-#endif
-
 	/* free the document */
 	xmlFreeDoc(doc);
 
@@ -299,7 +287,7 @@ static void increment_version(xmlNode *root_element)
 	xmlSetProp(root_element, BAD_CAST "config_version", BAD_CAST newver);
 }
 
-static xmlNode *findnode(xmlNode *root, char *name)
+static xmlNode *findnode(xmlNode *root, const char *name)
 {
 	xmlNode *cur_node;
 
@@ -333,7 +321,7 @@ static xmlChar *get_fence_type(xmlNode *clusternode, xmlNode **fencenode)
 }
 
 /* Check the fence type exists under <fencedevices> */
-static xmlNode *valid_fence_type(xmlNode *root, char *fencetype)
+static xmlNode *valid_fence_type(xmlNode *root, const char *fencetype)
 {
 	xmlNode *devs;
 	xmlNode *cur_node;
@@ -390,7 +378,7 @@ static xmlNode *find_multicast_addr(xmlNode *clusternodes)
 	return NULL;
 }
 
-static xmlNode *find_node(xmlNode *clusternodes, char *nodename)
+static xmlNode *find_node(xmlNode *clusternodes, const char *nodename)
 {
 	xmlNode *cur_node;
 
@@ -409,7 +397,7 @@ static xmlNode *find_node(xmlNode *clusternodes, char *nodename)
 /* Print name=value pairs for a (n XML) node.
  * "ignore" is a string to ignore if present as a property (probably already printed on the main line)
  */
-static int print_properties(xmlNode *node, char *prefix, char *ignore, char *ignore2)
+static int print_properties(xmlNode *node, const char *prefix, const char *ignore, const char *ignore2)
 {
 	xmlAttr *attr;
 	int done_prefix = 0;
@@ -436,17 +424,18 @@ static int print_properties(xmlNode *node, char *prefix, char *ignore, char *ign
 }
 
 /* Add name=value pairs from the commandline as properties to a node */
-static void add_fence_args(xmlNode *fencenode, int argc, char **argv, int optind)
+static void add_fence_args(xmlNode *fencenode, int argc, char **argv, int optindex)
 {
 	int i;
 
-	for (i = optind; i<argc; i++)
+	for (i = optindex; i<argc; i++)
 	{
 		char *prop;
 		char *value;
 		char *equals;
 
 		prop = strdup(argv[i]);
+		// FIXME: handle failed strdup
 		equals = strchr(prop, '=');
 		if (!equals)
 			die("option '%s' is not opt=value pair\n", prop);
@@ -466,7 +455,7 @@ static void add_fence_args(xmlNode *fencenode, int argc, char **argv, int optind
 }
 
 static void add_clusternode(xmlNode *root_element, struct option_info *ninfo,
-			    int argc, char **argv, int optind)
+			    int argc, char **argv, int optindex)
 {
 	xmlNode *clusternodes;
 	xmlNode *newnode;
@@ -491,7 +480,7 @@ static void add_clusternode(xmlNode *root_element, struct option_info *ninfo,
 		die("nodeid %s already in use\n", ninfo->nodeid);
 
         /* Don't allow random fence types */
-	if (!valid_fence_type(root_element, ninfo->fence_type))
+	if (ninfo->fence_type && !valid_fence_type(root_element, ninfo->fence_type))
 		die("fence type '%s' not known\n", ninfo->fence_type);
 
 	/* Add the new node */
@@ -511,19 +500,22 @@ static void add_clusternode(xmlNode *root_element, struct option_info *ninfo,
 	}
 
 	/* Add the fence attributes */
-	newfence = xmlNewNode(NULL, BAD_CAST "fence");
-	newfencemethod = xmlNewNode(NULL, BAD_CAST "method");
-	xmlSetProp(newfencemethod, BAD_CAST "name", BAD_CAST "single");
+	if (ninfo->fence_type)
+	{
+		newfence = xmlNewNode(NULL, BAD_CAST "fence");
+		newfencemethod = xmlNewNode(NULL, BAD_CAST "method");
+		xmlSetProp(newfencemethod, BAD_CAST "name", BAD_CAST "single");
 
-	newfencedevice = xmlNewNode(NULL, BAD_CAST "device");
-	xmlSetProp(newfencedevice, BAD_CAST "name", BAD_CAST ninfo->fence_type);
+		newfencedevice = xmlNewNode(NULL, BAD_CAST "device");
+		xmlSetProp(newfencedevice, BAD_CAST "name", BAD_CAST ninfo->fence_type);
 
-	/* Add name=value options */
-	add_fence_args(newfencedevice, argc, argv, optind+1);
+		/* Add name=value options */
+		add_fence_args(newfencedevice, argc, argv, optindex+1);
 
-	xmlAddChild(newnode, newfence);
-	xmlAddChild(newfence, newfencemethod);
-	xmlAddChild(newfencemethod, newfencedevice);
+		xmlAddChild(newnode, newfence);
+		xmlAddChild(newfence, newfencemethod);
+		xmlAddChild(newfencemethod, newfencedevice);
+	}
 }
 
 static xmlDoc *open_configfile(struct option_info *ninfo)
@@ -578,8 +570,6 @@ struct option addnode_options[] =
       { "fence_type", required_argument, NULL, 'f'},
       { "outputfile", required_argument, NULL, 'o'},
       { "configfile", required_argument, NULL, 'c'},
-      { "no_ccs", no_argument, NULL, 'C'},
-      { "force_ccs", no_argument, NULL, 'F'},
       { NULL, 0, NULL, 0 },
 };
 
@@ -587,8 +577,6 @@ struct option delnode_options[] =
 {
       { "outputfile", required_argument, NULL, 'o'},
       { "configfile", required_argument, NULL, 'c'},
-      { "no_ccs", no_argument, NULL, 'C'},
-      { "force_ccs", no_argument, NULL, 'F'},
       { NULL, 0, NULL, 0 },
 };
 
@@ -596,8 +584,6 @@ struct option addfence_options[] =
 {
       { "outputfile", required_argument, NULL, 'o'},
       { "configfile", required_argument, NULL, 'c'},
-      { "no_ccs", no_argument, NULL, 'C'},
-      { "force_ccs", no_argument, NULL, 'F'},
       { NULL, 0, NULL, 0 },
 };
 
@@ -618,6 +604,15 @@ struct option list_options[] =
       { NULL, 0, NULL, 0 },
 };
 
+struct option create_options[] =
+{
+      { "configfile", required_argument, NULL, 'c'},
+      { "nodes", required_argument, NULL, 'n'},
+      { "fence", required_argument, NULL, 'f'},
+      { "verbose", no_argument, NULL, 'v'},
+      { NULL, 0, NULL, 0 },
+};
+
 
 static int next_nodeid(int startid, int *nodeids, int nodecount)
 {
@@ -786,7 +781,6 @@ void add_node(int argc, char **argv)
 	xmlNode *root_element;
 
 	memset(&ninfo, 0, sizeof(ninfo));
-	ninfo.tell_ccsd = 1;
 	ninfo.votes = "1";
 
 	while ( (opt = getopt_long(argc, argv, "v:n:a:f:o:c:CFh?", addnode_options, NULL)) != EOF)
@@ -819,14 +813,6 @@ void add_node(int argc, char **argv)
 			ninfo.outputfile = strdup(optarg);
 			break;
 
-		case 'C':
-			ninfo.tell_ccsd = 0;
-			break;
-
-		case 'F':
-			ninfo.force_ccsd = 1;
-			break;
-
 		case '?':
 		default:
 			addnode_usage(argv[0]);
@@ -839,10 +825,6 @@ void add_node(int argc, char **argv)
 	else
 		addnode_usage(argv[0]);
 
-	if (!ninfo.fence_type)
-		addnode_usage(argv[0]);
-
-
 	doc = open_configfile(&ninfo);
 
 	root_element = xmlDocGetRootElement(doc);
@@ -866,7 +848,6 @@ void del_node(int argc, char **argv)
 	xmlNode *root_element;
 
 	memset(&ninfo, 0, sizeof(ninfo));
-	ninfo.tell_ccsd = 1;
 
 	while ( (opt = getopt_long(argc, argv, "o:c:CFh?", delnode_options, NULL)) != EOF)
 	{
@@ -880,14 +861,6 @@ void del_node(int argc, char **argv)
 			ninfo.outputfile = strdup(optarg);
 			break;
 
-		case 'C':
-			ninfo.tell_ccsd = 0;
-			break;
-
-		case 'F':
-			ninfo.force_ccsd = 1;
-			break;
-
 		case '?':
 		default:
 			delnode_usage(argv[0]);
@@ -995,22 +968,19 @@ void list_nodes(int argc, char **argv)
 
 void create_skeleton(int argc, char **argv)
 {
-	xmlNode *root_element;
-	xmlNode *fencedevices;
-	xmlNode *clusternodes;
-	xmlNode *rm;
-	xmlNode *rm1;
-	xmlNode *rm2;
-	xmlDocPtr doc;
+	char *fencename = NULL;
 	char *clustername;
 	struct option_info ninfo;
 	struct stat st;
+	FILE *outfile;
+	int i;
 	int twonode = 0;
+	int numnodes=0;
 	int opt;
 
 	memset(&ninfo, 0, sizeof(ninfo));
 
-	while ( (opt = getopt_long(argc, argv, "c:2h?", list_options, NULL)) != EOF)
+	while ( (opt = getopt_long(argc, argv, "c:2hn:f:?", create_options, NULL)) != EOF)
 	{
 		switch(opt)
 		{
@@ -1020,6 +990,13 @@ void create_skeleton(int argc, char **argv)
 
 		case '2':
 			twonode = 1;
+			numnodes = 2;
+			break;
+		case 'n':
+			numnodes = atoi(optarg);
+			break;
+		case 'f':
+			fencename = strdup(optarg);
 			break;
 
 		case '?':
@@ -1040,41 +1017,46 @@ void create_skeleton(int argc, char **argv)
 		die("%s already exists", ninfo.outputfile);
 
 	/* Init libxml */
-	xmlInitParser();
-	LIBXML_TEST_VERSION;
-
-	doc = xmlNewDoc(BAD_CAST "1.0");
-	root_element = xmlNewNode(NULL, BAD_CAST "cluster");
-	xmlDocSetRootElement(doc, root_element);
-
-	xmlSetProp(root_element, BAD_CAST "name", BAD_CAST clustername);
-	xmlSetProp(root_element, BAD_CAST "config_version", BAD_CAST "1");
-
-	/* Generate extra bits for a 2node cman cluster */
-	if (twonode) {
-
-		xmlNode *cman = xmlNewNode(NULL, BAD_CAST "cman");
-		xmlSetProp(cman, BAD_CAST "two_node", BAD_CAST "1");
-		xmlSetProp(cman, BAD_CAST "expected_votes", BAD_CAST "1");
-		xmlAddChild(root_element, cman);
+	outfile = fopen(ninfo.outputfile, "w+");
+	if (!outfile) {
+		perror(" Can't open output file");
+		return;
 	}
 
-	clusternodes = xmlNewNode(NULL, BAD_CAST "clusternodes");
-	fencedevices = xmlNewNode(NULL, BAD_CAST "fencedevices");
-	rm = xmlNewNode(NULL, BAD_CAST "rm");
-	rm1 = xmlNewNode(NULL, BAD_CAST "failoverdomains");
+	fprintf(outfile, "<?xml version=\"1.0\"?>\n");
+	fprintf(outfile, "<cluster name=\"%s\" config_version=\"1\">\n", clustername);
+	fprintf(outfile, "\n");
+	if (twonode) {
+		fprintf(outfile, "  <cman two_node=\"1\" expected_votes=\"1\"/>\n");
+	}
 
-	xmlAddChild(root_element, clusternodes);
-	xmlAddChild(root_element, fencedevices);
-	xmlAddChild(root_element, rm);
-
-	/* Create empty resource manager sections to keep GUI happy */
-	rm2 = xmlNewNode(NULL, BAD_CAST "resources");
-	xmlAddChild(rm, rm1);
-	xmlAddChild(rm, rm2);
-
-	save_file(doc, &ninfo);
+	fprintf(outfile, "  <clusternodes>\n");
+	for (i=1; i< numnodes; i++) {
+		fprintf(outfile, "    <clusternode name=\"NEEDNAME-%02d\" votes=\"1\" nodeid=\"%d\">\n", i, i);
+		fprintf(outfile, "      <fence>\n");
+		fprintf(outfile, "        <method name=\"single\">\n");
+		if (fencename) {
+			fprintf(outfile, "          <device name=\"fence1\" ADDARGS/>\n");
+		}
+		fprintf(outfile, "        </method>\n");
+		fprintf(outfile, "      </fence>\n");
+		fprintf(outfile, "    </clusternode>\n");
+	}
+	fprintf(outfile, "  </clusternodes>\n");
+	fprintf(outfile, "\n");
+	fprintf(outfile, "  <fencedevices>\n");
+	if (fencename) {
+		fprintf(outfile, "    <fencedevice name=\"fence1\" agent=\"%s\" ADDARGS/>\n", fencename);
+	}
+	fprintf(outfile, "  </fencedevices>\n");
+	fprintf(outfile, "\n");
+	fprintf(outfile, "  <rm>\n");
+	fprintf(outfile, "    <failoverdomains/>\n");
+	fprintf(outfile, "    <resources/>\n");
+	fprintf(outfile, "  </rm>\n");
+	fprintf(outfile, "</cluster>\n");
 
+	fclose(outfile);
 }
 
 void add_fence(int argc, char **argv)
@@ -1089,7 +1071,6 @@ void add_fence(int argc, char **argv)
 	int opt;
 
 	memset(&ninfo, 0, sizeof(ninfo));
-	ninfo.tell_ccsd = 1;
 
 	while ( (opt = getopt_long(argc, argv, "c:o:CFh?", list_options, NULL)) != EOF)
 	{
@@ -1102,14 +1083,6 @@ void add_fence(int argc, char **argv)
 			ninfo.outputfile = strdup(optarg);
 			break;
 
-		case 'C':
-			ninfo.tell_ccsd = 0;
-			break;
-
-		case 'F':
-			ninfo.force_ccsd = 1;
-			break;
-
 		case '?':
 		default:
 			addfence_usage(argv[0]);
@@ -1160,7 +1133,6 @@ void del_fence(int argc, char **argv)
 	int opt;
 
 	memset(&ninfo, 0, sizeof(ninfo));
-	ninfo.tell_ccsd = 1;
 
 	while ( (opt = getopt_long(argc, argv, "c:o:CFhv?", list_options, NULL)) != EOF)
 	{
@@ -1173,14 +1145,6 @@ void del_fence(int argc, char **argv)
 			ninfo.outputfile = strdup(optarg);
 			break;
 
-		case 'C':
-			ninfo.tell_ccsd = 0;
-			break;
-
-		case 'F':
-			ninfo.force_ccsd = 1;
-			break;
-
 		case '?':
 		default:
 			delfence_usage(argv[0]);
