@@ -191,9 +191,9 @@ struct fd {
 int setup_ccs(void);
 void close_ccs(void);
 void reread_ccs(void);
-void read_ccs_name(char *path, char *name);
-void read_ccs_yesno(char *path, int *yes, int *no);
-void read_ccs_int(char *path, int *config_val);
+void read_ccs_name(const char *path, char *name);
+void read_ccs_yesno(const char *path, int *yes, int *no);
+void read_ccs_int(const char *path, int *config_val);
 int read_ccs(struct fd *fd);
 
 /* cpg.c */
@@ -203,6 +203,7 @@ int setup_cpg(void);
 void close_cpg(void);
 int set_protocol(void);
 void free_cg(struct change *cg);
+void node_history_init(struct fd *fd, int nodeid);
 void node_history_fence(struct fd *fd, int victim, int master, int how,
 			uint64_t mastertime);
 void send_external(struct fd *fd, int victim);
@@ -265,7 +266,7 @@ void fence_victims(struct fd *fd);
 /* logging.c */
 
 void init_logging(void);
-void setup_logging();
+void setup_logging(void);
 void close_logging(void);
 
 #endif				/*  __FD_DOT_H__  */
