@@ -13,6 +13,7 @@ static struct pollfd *pollfd = NULL;
 static pthread_t query_thread;
 static pthread_mutex_t query_mutex;
 static struct list_head controlled_entries;
+static char default_name[8];
 
 struct client {
 	int fd;
@@ -25,7 +26,7 @@ static int do_read(int fd, void *buf, size_t count)
 	int rv, off = 0;
 
 	while (off < count) {
-		rv = read(fd, buf + off, count - off);
+		rv = read(fd, (char *)buf + off, count - off);
 		if (rv == 0)
 			return -1;
 		if (rv == -1 && errno == EINTR)
@@ -42,7 +43,7 @@ static int do_write(int fd, void *buf, size_t count)
 	int rv, off = 0;
 
  retry:
-	rv = write(fd, buf + off, count);
+	rv = write(fd, (char *)buf + off, count);
 	if (rv == -1 && errno == EINTR)
 		goto retry;
 	if (rv < 0) {
@@ -335,7 +336,7 @@ static void query_node_info(int f, int data_nodeid)
 	struct fenced_node node;
 	int nodeid, rv;
 
-	fd = find_fd("default");
+	fd = find_fd(default_name);
 	if (!fd) {
 		rv = -ENOENT;
 		goto out;
@@ -360,7 +361,7 @@ static void query_domain_info(int f)
 	struct fenced_domain domain;
 	int rv;
 
-	fd = find_fd("default");
+	fd = find_fd(default_name);
 	if (!fd) {
 		rv = -ENOENT;
 		goto out;
@@ -384,7 +385,7 @@ static void query_domain_nodes(int f, int option, int max)
 	struct fenced_node *nodes = NULL;
 	int rv, result;
 
-	fd = find_fd("default");
+	fd = find_fd(default_name);
 	if (!fd) {
 		result = -ENOENT;
 		node_count = 0;
@@ -426,6 +427,7 @@ static void process_connection(int ci)
 	char *extra = NULL;
 	int rv, extra_len;
 
+
 	rv = do_read(client[ci].fd, &h, sizeof(h));
 	if (rv < 0) {
 		log_debug("connection %d read error %d", ci, rv);
@@ -460,13 +462,13 @@ static void process_connection(int ci)
 
 	switch (h.command) {
 	case FENCED_CMD_JOIN:
-		do_join("default");
+		do_join(default_name);
 		break;
 	case FENCED_CMD_LEAVE:
-		do_leave("default");
+		do_leave(default_name);
 		break;
 	case FENCED_CMD_EXTERNAL:
-		do_external("default", extra, extra_len);
+		do_external(default_name, extra, extra_len);
 		break;
 	case FENCED_CMD_DUMP_DEBUG:
 	case FENCED_CMD_NODE_INFO:
@@ -499,7 +501,7 @@ static void process_listener(int ci)
 	log_debug("client connection %d fd %d", i, fd);
 }
 
-static int setup_listener(char *sock_path)
+static int setup_listener(const char *sock_path)
 {
 	struct sockaddr_un addr;
 	socklen_t addrlen;
@@ -555,14 +557,14 @@ static void *process_queries(void *arg)
 
 	rv = setup_listener(FENCED_QUERY_SOCK_PATH);
 	if (rv < 0)
-		exit (-1);
+		return NULL;
 
 	s = rv;
 
 	for (;;) {
 		f = accept(s, NULL, NULL);
 		if (f < 0)
-			exit (-1);
+			return NULL;
 
 		rv = do_read(f, &h, sizeof(h));
 		if (rv < 0) {
@@ -621,7 +623,7 @@ struct controlled_entry {
 	char path[PATH_MAX+1];
 };
 
-static void register_controlled_dir(char *path)
+static void register_controlled_dir(const char *path)
 {
 	struct controlled_entry *ce;
 
@@ -1025,8 +1027,9 @@ int main(int argc, char **argv)
 	INIT_LIST_HEAD(&domains);
 	INIT_LIST_HEAD(&controlled_entries);
 
+	strcpy(default_name, "default");
+
 	read_arguments(argc, argv);
-	lockfile();
 
 	if (!daemon_debug_opt) {
 		if (daemon(0, 0) < 0) {
@@ -1035,13 +1038,15 @@ int main(int argc, char **argv)
 		}
 		umask(0);
 	}
+	lockfile();
 	init_logging();
-	log_level(LOG_INFO, "fenced %s", RELEASE_VERSION);
+	log_level(LOG_INFO, "fenced %s started", RELEASE_VERSION);
 	signal(SIGTERM, sigterm_handler);
 	set_oom_adj(-16);
 
 	loop();
 
+	unlink(LOCKFILE_NAME);
 	return 0;
 }
 
