@@ -1,16 +1,3 @@
-/******************************************************************************
-*******************************************************************************
-**
-**  Copyright (C) Sistina Software, Inc.  1997-2003  All rights reserved.
-**  Copyright (C) 2004-2007 Red Hat, Inc.  All rights reserved.
-**
-**  This copyrighted material is made available to anyone wishing to use,
-**  modify, copy, or redistribute it subject to the terms and conditions
-**  of the GNU General Public License v.2.
-**
-*******************************************************************************
-******************************************************************************/
-
 #include "util.h"
 #include "file.h"
 #include "rgrp.h"
@@ -25,7 +12,7 @@
 #define IS_LEAF     (1)
 #define IS_DINODE   (2)
 
-#define dir_hash(qstr) (gfs_dir_hash((qstr)->name, (qstr)->len))
+#define dir_hash(qstr) (gfs_dir_hash((char *)(qstr)->name, (qstr)->len))
 
 /* Detect directory is a stuffed inode */
 int fsck_inode_is_stuffed(struct fsck_inode *ip)
@@ -77,7 +64,7 @@ int dirent_first(osi_buf_t *bh, struct gfs_dirent **dent)
  *
  * Returns: 0 on success, error code otherwise
  */
-int dirent_next(osi_buf_t *bh, struct gfs_dirent **dent)
+static int dirent_next(osi_buf_t *bh, struct gfs_dirent **dent)
 {
 	struct gfs_dirent *tmp, *cur;
 	char *bh_end;
@@ -221,18 +208,18 @@ static int get_leaf(struct fsck_inode *dip, uint64 leaf_no, osi_buf_t **bhp)
 /**
  * get_first_leaf - Get first leaf
  * @dip: The GFS inode
- * @index:
+ * @lindex:
  * @bh_out:
  *
  * Returns: 0 on success, error code otherwise
  */
 
-static int get_first_leaf(struct fsck_inode *dip, uint32 index, osi_buf_t **bh_out)
+static int get_first_leaf(struct fsck_inode *dip, uint32 lindex, osi_buf_t **bh_out)
 {
 	uint64 leaf_no;
 	int error;
 
-	error = get_leaf_nr(dip, index, &leaf_no);
+	error = get_leaf_nr(dip, lindex, &leaf_no);
 	if (!error)
 		error = get_leaf(dip, leaf_no, bh_out);
 
@@ -369,7 +356,7 @@ static int linked_leaf_search(struct fsck_inode *dip, identifier_t *id,
 			      struct gfs_dirent **dent_prev, osi_buf_t **bh_out)
 {
 	osi_buf_t *bh = NULL, *bh_next;
-	uint32 hsize, index;
+	uint32 hsize, lindex;
 	uint32 hash;
 	int error = 0;
 
@@ -383,9 +370,9 @@ static int linked_leaf_search(struct fsck_inode *dip, identifier_t *id,
 
 	if(id->type == ID_FILENAME){
 		hash = dir_hash(id->filename);
-		index = hash >> (32 - dip->i_di.di_depth);
+		lindex = hash >> (32 - dip->i_di.di_depth);
 
-		error = get_first_leaf(dip, index, &bh_next);
+		error = get_first_leaf(dip, lindex, &bh_next);
 		if (error){
 			return error;
 		}
@@ -416,8 +403,8 @@ static int linked_leaf_search(struct fsck_inode *dip, identifier_t *id,
 
 		relse_buf(dip->i_sbd, bh);
 	} else if(id->type == ID_INUM){
-		for(index=0; index < (1 << dip->i_di.di_depth); index++){
-			error = get_first_leaf(dip, index, &bh_next);
+		for(lindex=0; lindex < (1 << dip->i_di.di_depth); lindex++){
+			error = get_first_leaf(dip, lindex, &bh_next);
 			if (error){
 				return error;
 			}
@@ -464,7 +451,7 @@ static int linked_leaf_search(struct fsck_inode *dip, identifier_t *id,
  */
 static int dir_e_search(struct fsck_inode *dip, identifier_t *id, unsigned int *type)
 {
-	osi_buf_t *bh;
+	osi_buf_t *bh = NULL;
 	struct gfs_dirent *dent;
 	int error;
 
@@ -562,6 +549,7 @@ static int dir_l_search(struct fsck_inode *dip, identifier_t *id, unsigned int *
 			exit(1);
 		}
 		id->inum = (struct gfs_inum *)malloc(sizeof(struct gfs_inum));
+		// FIXME: don't dereference NULL on failed malloc
 		memset(id->inum, 0, sizeof(struct gfs_inum));
 
 		gfs_inum_in(id->inum, (char *)&dent->de_inum);
@@ -571,10 +559,12 @@ static int dir_l_search(struct fsck_inode *dip, identifier_t *id, unsigned int *
 			exit(1);
 		}
 		id->filename = (osi_filename_t *)malloc(sizeof(osi_filename_t));
+		// FIXME: don't dereference NULL on failed malloc
 		memset(id->filename, 0, sizeof(osi_filename_t));
 
 		id->filename->len = gfs16_to_cpu(dent->de_name_len);
 		id->filename->name = malloc(id->filename->len);
+		// FIXME: don't dereference NULL on failed malloc
 		memset(id->filename->name, 0, id->filename->len);
 
 		memcpy(id->filename->name, (char *)dent+sizeof(struct gfs_dirent),
@@ -742,12 +732,12 @@ static int dir_make_exhash(struct fsck_inode *dip)
 /**
  * dir_split_leaf - Split a leaf block into two
  * @dip: The GFS inode
- * @index:
+ * @lindex:
  * @leaf_no:
  *
  * Returns: 0 on success, error code on failure
  */
-static int dir_split_leaf(struct fsck_inode *dip, uint32 index, uint64 leaf_no)
+static int dir_split_leaf(struct fsck_inode *dip, uint32 lindex, uint64 leaf_no)
 {
 	struct fsck_sb *sdp = dip->i_sbd;
 	osi_buf_t *nbh, *obh, *dibh;
@@ -805,7 +795,7 @@ static int dir_split_leaf(struct fsck_inode *dip, uint32 index, uint64 leaf_no)
 	}
 	half_len = len >> 1;
 
-	start = (index & ~(len - 1));
+	start = (lindex & ~(len - 1));
 
 	log_debug("Splitting leaf: len = %u, half_len = %u\n", len, half_len);
 
@@ -814,6 +804,7 @@ static int dir_split_leaf(struct fsck_inode *dip, uint32 index, uint64 leaf_no)
 	    This code is complicated enough already.  */
 
 	lp = (uint64 *)malloc(half_len * sizeof(uint64));
+	// FIXME: don't dereference NULL on failed malloc
 	memset(lp, 0, half_len * sizeof(uint64));
 
 	error = readi(dip, (char *)lp, start * sizeof(uint64),
@@ -1080,7 +1071,7 @@ static int dir_double_exhash(struct fsck_inode *dip)
 
 
 static int dir_e_del(struct fsck_inode *dip, osi_filename_t *filename){
-	int index;
+	int lindex;
 	int error;
 	int found = 0;
 	uint64 leaf_no;
@@ -1092,10 +1083,10 @@ static int dir_e_del(struct fsck_inode *dip, osi_filename_t *filename){
 	id.filename = filename;
 	id.inum = NULL;
 
-	index = (1 << (dip->i_di.di_depth))-1;
+	lindex = (1 << (dip->i_di.di_depth))-1;
 
-	for(; (index >= 0) && !found; index--){
-		error = get_leaf_nr(dip, index, &leaf_no);
+	for(; (lindex >= 0) && !found; lindex--){
+		error = get_leaf_nr(dip, lindex, &leaf_no);
 		if (error){
 			log_err("dir_e_del:  Unable to get leaf number.\n");
 			return error;
@@ -1239,7 +1230,7 @@ static int dir_e_add(struct fsck_inode *dip, osi_filename_t *filename,
 	osi_buf_t *bh, *nbh, *dibh;
 	struct gfs_leaf *leaf, *nleaf;
 	struct gfs_dirent *dent;
-	uint32 hsize, index;
+	uint32 hsize, lindex;
 	uint32 hash;
 	uint64 leaf_no, bn;
 	int error;
@@ -1257,10 +1248,10 @@ static int dir_e_add(struct fsck_inode *dip, osi_filename_t *filename,
 	/*  Figure out the address of the leaf node.  */
 
 	hash = dir_hash(filename);
-	index = hash >> (32 - dip->i_di.di_depth);
+	lindex = hash >> (32 - dip->i_di.di_depth);
 
 
-	error = get_leaf_nr(dip, index, &leaf_no);
+	error = get_leaf_nr(dip, lindex, &leaf_no);
 	if (error){
 		log_err("dir_e_add:  Unable to get leaf number.\n");
 		return error;
@@ -1284,7 +1275,7 @@ static int dir_e_add(struct fsck_inode *dip, osi_filename_t *filename,
 				/*  Can we split the leaf?  */
 				relse_buf(sdp, bh);
 
-				error = dir_split_leaf(dip, index, leaf_no);
+				error = dir_split_leaf(dip, lindex, leaf_no);
 				if (error){
 					log_err("dir_e_add:  Unable to split leaf.\n");
 					return error;
@@ -1607,20 +1598,20 @@ int fs_dirent_alloc(struct fsck_inode *dip, osi_buf_t *bh,
 
 
 /**
- * get_leaf_nr - Get a leaf number associated with the index
+ * get_leaf_nr - Get a leaf number associated with the lindex
  * @dip: The GFS inode
- * @index:
+ * @lindex:
  * @leaf_out:
  *
  * Returns: 0 on success, error code otherwise
  */
 
-int get_leaf_nr(struct fsck_inode *dip, uint32 index, uint64 *leaf_out)
+int get_leaf_nr(struct fsck_inode *dip, uint32 lindex, uint64 *leaf_out)
 {
 	uint64 leaf_no;
 	int error = -1;
 	error = readi(dip, (char *)&leaf_no,
-		      index * sizeof(uint64), sizeof(uint64));
+		      lindex * sizeof(uint64), sizeof(uint64));
 	if (error != sizeof(uint64)){
 		log_debug("get_leaf_nr:  Bad internal read.  (rtn = %d)\n",
 			  error);
@@ -1634,15 +1625,15 @@ int get_leaf_nr(struct fsck_inode *dip, uint32 index, uint64 *leaf_out)
 
 
 /**
- * put_leaf_nr - Put a leaf number associated with the index
+ * put_leaf_nr - Put a leaf number associated with the lindex
  * @dip: The GFS inode
- * @index:
+ * @lindex:
  * @leaf_out:
  *
  * Returns: 0 on success, error code otherwise
  */
 
-int put_leaf_nr(struct fsck_inode *dip, uint32 index, uint64 leaf_out)
+int put_leaf_nr(struct fsck_inode *dip, uint32 lindex, uint64 leaf_out)
 {
 	uint64 leaf_no;
 	int error = -1;
@@ -1650,7 +1641,7 @@ int put_leaf_nr(struct fsck_inode *dip, uint32 index, uint64 leaf_out)
 	leaf_no = cpu_to_gfs64(leaf_out);
 
 	error = writei(dip, (char *)&leaf_no,
-		       index * sizeof(uint64), sizeof(uint64));
+		       lindex * sizeof(uint64), sizeof(uint64));
 	if (error != sizeof(uint64)){
 		log_debug("put_leaf_nr:  Bad internal write.  (rtn = %d)\n",
 			  error);
@@ -1739,8 +1730,7 @@ int dirent_repair(struct fsck_inode *ip, osi_buf_t *bh, struct gfs_dirent *de,
 				sizeof(struct gfs_dinode);
 		else
 			de->de_rec_len = BH_SIZE(bh) - sizeof(struct gfs_leaf);
-	}
-	else {
+	} else {
 		bh_end = BH_DATA(bh) + BH_SIZE(bh);
 		/* first, figure out a probable name length */
 		p = (char *)dent + sizeof(struct gfs_dirent);
@@ -1763,3 +1753,23 @@ int dirent_repair(struct fsck_inode *ip, osi_buf_t *bh, struct gfs_dirent *de,
 	write_buf(ip->i_sbd, bh, 0);
 	return 0;
 }
+
+/**
+ * dirblk_truncate - truncate a directory block
+ */
+void dirblk_truncate(struct fsck_inode *ip, struct gfs_dirent *fixb,
+		     osi_buf_t *bh)
+{
+	char *bh_end;
+	struct gfs_dirent de;
+	uint16_t old_rec_len;
+
+	bh_end = BH_DATA(bh) + BH_SIZE(bh);
+	/* truncate the block to save the most dentries.  To do this we
+	   have to patch the previous dent. */
+	gfs_dirent_in(&de, (char *)fixb);
+	old_rec_len = de.de_rec_len;
+	de.de_rec_len = bh_end - (char *)fixb;
+	gfs_dirent_out(&de, (char *)fixb);
+	write_buf(ip->i_sbd, bh, 0);
+}
