@@ -1,16 +1,3 @@
-/******************************************************************************
-*******************************************************************************
-**
-**  Copyright (C) Sistina Software, Inc.  1997-2003  All rights reserved.
-**  Copyright (C) 2004-2005 Red Hat, Inc.  All rights reserved.
-**
-**  This copyrighted material is made available to anyone wishing to use,
-**  modify, copy, or redistribute it subject to the terms and conditions
-**  of the GNU General Public License v.2.
-**
-*******************************************************************************
-******************************************************************************/
-
 #include "stdio.h"
 #include "fsck_incore.h"
 #include "fsck.h"
@@ -19,10 +6,19 @@
 #include "inode_hash.h"
 #include "inode.h"
 #include "lost_n_found.h"
+#include "metawalk.h"
+
+struct metawalk_fxns pass4_fxns_delete = {
+	.private = NULL,
+	.check_metalist = delete_metadata,
+	.check_data = delete_data,
+	.check_eattr_indir = delete_eattr_indir,
+	.check_eattr_leaf = delete_eattr_leaf,
+};
 
 /* Updates the link count of an inode to what the fsck has seen for
  * link count */
-int fix_inode_count(struct fsck_sb *sbp, struct inode_info *ii,
+static int fix_inode_count(struct fsck_sb *sbp, struct inode_info *ii,
 		    struct fsck_inode *ip)
 {
 	log_info("Fixing inode count for %"PRIu64"\n",
@@ -39,7 +35,7 @@ int fix_inode_count(struct fsck_sb *sbp, struct inode_info *ii,
 	return 0;
 }
 
-int scan_inode_list(struct fsck_sb *sbp, osi_list_t *list) {
+static int scan_inode_list(struct fsck_sb *sbp, osi_list_t *list) {
 	osi_list_t *tmp;
 	struct inode_info *ii;
 	struct fsck_inode *ip;
@@ -49,6 +45,8 @@ int scan_inode_list(struct fsck_sb *sbp, osi_list_t *list) {
 	/* FIXME: should probably factor this out into a generic
 	 * scanning fxn */
 	osi_list_foreach(tmp, list) {
+		if (skip_this_pass || fsck_abort) /* if asked to skip the rest */
+			return 0;
 		if(!(ii = osi_list_entry(tmp, struct inode_info, list))) {
 			log_crit("osi_list_foreach broken in scan_info_list!!\n");
 			exit(1);
@@ -72,12 +70,18 @@ int scan_inode_list(struct fsck_sb *sbp, osi_list_t *list) {
 				log_err("Unlinked inode contains"
 					"bad blocks\n",
 					ii->inode);
-				if(query(sbp, "Clear unlinked inode with bad blocks? (y/n) ")) {
-					block_set(sbp->bl, ii->inode, block_free);
+				if(query(sbp, "Delete unlinked inode with bad "
+					 "blocks? (y/n) ")) {
+					load_inode(sbp, ii->inode, &ip);
+					check_inode_eattr(ip,
+							  &pass4_fxns_delete);
+					check_metatree(ip, &pass4_fxns_delete);
+					free_inode(&ip);
+					block_set(sbp->bl, ii->inode,
+						  block_free);
 					continue;
-				} else {
+				} else
 					log_err("Unlinked inode with bad blocks not cleared\n");
-				}
 			}
 			if(q.block_type != inode_dir &&
 			   q.block_type != inode_file &&
@@ -86,9 +90,23 @@ int scan_inode_list(struct fsck_sb *sbp, osi_list_t *list) {
 			   q.block_type != inode_chr &&
 			   q.block_type != inode_fifo &&
 			   q.block_type != inode_sock) {
-				log_err("Unlinked block marked as inode not an inode\n");
-				block_set(sbp->bl, ii->inode, block_free);
-				log_err("Cleared\n");
+				log_err("Unlinked block marked as an inode is "
+					"not an inode (%d)\n", q.block_type);
+				if(query(sbp, "Delete unlinked inode"
+					 "? (y/n) ")) {
+					if (!load_inode(sbp, ii->inode, &ip)) {
+						check_inode_eattr(ip,
+							   &pass4_fxns_delete);
+						check_metatree(ip,
+							   &pass4_fxns_delete);
+					}
+					block_set(sbp->bl, ii->inode,
+						  block_free);
+					free_inode(&ip);
+					log_err("The inode was deleted\n");
+				} else
+					log_err("The inode was not deleted\n");
+
 				continue;
 			}
 			if(load_inode(sbp, ii->inode, &ip)) {
@@ -117,9 +135,8 @@ int scan_inode_list(struct fsck_sb *sbp, osi_list_t *list) {
 					fix_inode_count(sbp, ii, ip);
 					lf_addition = 1;
 				}
-			} else {
+			} else
 				log_err("Unlinked inode left unlinked\n");
-			}
 			free_inode(&ip);
 		}
 		else if(ii->link_count != ii->counted_links) {
@@ -154,7 +171,6 @@ int scan_inode_list(struct fsck_sb *sbp, osi_list_t *list) {
 		}
 	}
 
-
 	return 0;
 }
 
@@ -176,6 +192,8 @@ int pass4(struct fsck_sb *sbp, struct options *opts)
 			  sbp->lf_dip->i_di.di_entries);
 	log_info("Checking inode reference counts.\n");
 	for (i = 0; i < FSCK_HASH_SIZE; i++) {
+		if (skip_this_pass || fsck_abort) /* if asked to skip the rest */
+			return 0;
 		list = &sbp->inode_hash[i];
 		if(scan_inode_list(sbp, list)) {
 			stack;
