@@ -37,6 +37,7 @@ int fs_compute_bitstructs(struct fsck_rgrp *rgd)
 		return -1;
 	}
 	if(!memset(rgd->rd_bits, 0, length * sizeof(fs_bitmap_t))) {
+		free(rgd->rd_bits);
 		log_err("Unable to zero bitmap structure\n");
 		stack;
 		return -1;
@@ -78,6 +79,7 @@ int fs_compute_bitstructs(struct fsck_rgrp *rgd)
 	if(bytes_left){
 		log_err( "fs_compute_bitstructs:  Too many blocks in rgrp to "
 			"fit into available bitmap.\n");
+		free(rgd->rd_bits);
 		return -1;
 	}
 
@@ -93,6 +95,7 @@ int fs_compute_bitstructs(struct fsck_rgrp *rgd)
 			rgd->rd_bits[length - 1].bi_len,
 			GFS_NBBY,
 			rgd->rd_ri.ri_data);
+		free(rgd->rd_bits);
 		return -1;
 	}
 
@@ -100,11 +103,13 @@ int fs_compute_bitstructs(struct fsck_rgrp *rgd)
 	if(!(rgd->rd_bh = (osi_buf_t **)malloc(length * sizeof(osi_buf_t *)))) {
 		log_err("Unable to allocate osi_buf structure\n");
 		stack;
+		free(rgd->rd_bits);
 		return -1;
 	}
 	if(!memset(rgd->rd_bh, 0, length * sizeof(osi_buf_t *))) {
 		log_err("Unable to zero osi_buf structure\n");
 		stack;
+		free(rgd->rd_bits);
 		return -1;
 	}
 
@@ -150,7 +155,6 @@ int fs_rgrp_read(struct fsck_rgrp *rgd, int repair_if_corrupted)
 	int error;
 
 	if(rgd->rd_open_count){
-		log_debug("rgrp already read...\n");
 		rgd->rd_open_count++;
 		return 0;
 	}
@@ -231,7 +235,7 @@ void fs_rgrp_relse(struct fsck_rgrp *rgd)
 
 	rgd->rd_open_count--;
 	if(rgd->rd_open_count){
-		log_debug("rgrp still held...\n");
+		;
 	} else {
 		for (x = 0; x < length; x++){
 			if (rgd->rd_bh[x]) {
@@ -321,7 +325,9 @@ int fs_rgrp_recount(struct fsck_rgrp *rgd){
 		count[i] = 0;
 		for(j = 0; j < length; j++){
 			bits = &rgd->rd_bits[j];
-			count[i] += fs_bitcount(BH_DATA(rgd->rd_bh[j]) + bits->bi_offset,
+			count[i] += fs_bitcount((unsigned char *)
+						BH_DATA(rgd->rd_bh[j]) +
+						bits->bi_offset,
 						bits->bi_len, i);
 		}
 	}
