@@ -1,16 +1,3 @@
-/******************************************************************************
-*******************************************************************************
-**
-**  Copyright (C) Sistina Software, Inc.  1997-2003  All rights reserved.
-**  Copyright (C) 2004-2005 Red Hat, Inc.  All rights reserved.
-**
-**  This copyrighted material is made available to anyone wishing to use,
-**  modify, copy, or redistribute it subject to the terms and conditions
-**  of the GNU General Public License v.2.
-**
-*******************************************************************************
-******************************************************************************/
-
 #ifndef HELPER_PROGRAM
 
 #include "libgfs.h"
@@ -50,7 +37,7 @@
  */
 
 static void
-print_array(char *title, char *buf, int count)
+print_array(const char *title, char *buf, int count)
 {
 	int x;
 
@@ -345,20 +332,20 @@ gfs_jindex_print(struct gfs_jindex *ji)
  */
 
 void
-gfs_rindex_in(struct gfs_rindex *rindex, char *buf)
+gfs_rindex_in(struct gfs_rindex *grindex, char *buf)
 {
 	struct gfs_rindex *str = (struct gfs_rindex *) buf;
 
-	CPIN_64(rindex, str, ri_addr);
-	CPIN_32(rindex, str, ri_length);
-	CPIN_32(rindex, str, ri_pad);
+	CPIN_64(grindex, str, ri_addr);
+	CPIN_32(grindex, str, ri_length);
+	CPIN_32(grindex, str, ri_pad);
 
-	CPIN_64(rindex, str, ri_data1);
-	CPIN_32(rindex, str, ri_data);
+	CPIN_64(grindex, str, ri_data1);
+	CPIN_32(grindex, str, ri_data);
 
-	CPIN_32(rindex, str, ri_bitbytes);
+	CPIN_32(grindex, str, ri_bitbytes);
 
-	CPIN_08(rindex, str, ri_reserved, 64);
+	CPIN_08(grindex, str, ri_reserved, 64);
 
 }
 
@@ -370,20 +357,20 @@ gfs_rindex_in(struct gfs_rindex *rindex, char *buf)
  */
 
 void
-gfs_rindex_out(struct gfs_rindex *rindex, char *buf)
+gfs_rindex_out(struct gfs_rindex *grindex, char *buf)
 {
 	struct gfs_rindex *str = (struct gfs_rindex *) buf;
 
-	CPOUT_64(rindex, str, ri_addr);
-	CPOUT_32(rindex, str, ri_length);
-	CPOUT_32(rindex, str, ri_pad);
+	CPOUT_64(grindex, str, ri_addr);
+	CPOUT_32(grindex, str, ri_length);
+	CPOUT_32(grindex, str, ri_pad);
 
-	CPOUT_64(rindex, str, ri_data1);
-	CPOUT_32(rindex, str, ri_data);
+	CPOUT_64(grindex, str, ri_data1);
+	CPOUT_32(grindex, str, ri_data);
 
-	CPOUT_32(rindex, str, ri_bitbytes);
+	CPOUT_32(grindex, str, ri_bitbytes);
 
-	CPOUT_08(rindex, str, ri_reserved, 64);
+	CPOUT_08(grindex, str, ri_reserved, 64);
 
 }
 
@@ -800,7 +787,7 @@ gfs_dirent_out(struct gfs_dirent *dirent, char *buf)
  */
 
 void
-gfs_dirent_print(struct gfs_dirent *de, char *name)
+gfs_dirent_print(struct gfs_dirent *de, const char *name)
 {
 	char buf[GFS_FNAMESIZE + 1];
 
