@@ -114,7 +114,7 @@ static void info_bswap_in(struct dlm_plock_info *i)
 	i->owner	= le64_to_cpu(i->owner);
 }
 
-static char *op_str(int optype)
+static const char *op_str(int optype)
 {
 	switch (optype) {
 	case DLM_PLOCK_OP_LOCK:
@@ -128,7 +128,7 @@ static char *op_str(int optype)
 	}
 }
 
-static char *ex_str(int optype, int ex)
+static const char *ex_str(int optype, int ex)
 {
 	if (optype == DLM_PLOCK_OP_UNLOCK || optype == DLM_PLOCK_OP_GET)
 		return "-";
@@ -186,6 +186,15 @@ int setup_plocks(void)
 	return plock_device_fd;
 }
 
+void close_plocks(void)
+{
+	if (system_ckpt_handle)
+		saCkptFinalize(system_ckpt_handle);
+	if (plock_device_fd > 0)
+		close(plock_device_fd);
+
+}
+
 static uint32_t mg_to_ls_id(uint32_t fsid)
 {
 	struct lockspace *ls;
@@ -723,6 +732,7 @@ static void do_get(struct lockspace *ls, struct dlm_plock_info *in,
 		rv = 0;
 
 	write_result(ls, in, rv);
+	put_resource(r);
 }
 
 static void save_message(struct lockspace *ls, struct dlm_header *hd, int len,
@@ -1251,7 +1261,7 @@ static void _receive_sync(struct lockspace *ls, struct dlm_header *hd, int len)
 	}
 
 	if (hd->type == DLM_MSG_PLOCK_SYNC_LOCK)
-		add_lock(r, info.nodeid, info.owner, info.pid, !info.ex, 
+		add_lock(r, info.nodeid, info.owner, info.pid, info.ex, 
 			 info.start, info.end);
 	else if (hd->type == DLM_MSG_PLOCK_SYNC_WAITER)
 		add_waiter(ls, r, &info);
@@ -1398,6 +1408,7 @@ int limit_plocks(void)
 			return 2;
 		}
 		plock_rate_last = now;
+		plock_read_count++;
 	}
 	return 0;
 }
@@ -1534,11 +1545,6 @@ void process_saved_plocks(struct lockspace *ls)
 	}
 }
 
-void plock_exit(void)
-{
-	saCkptFinalize(system_ckpt_handle);
-}
-
 /* locks still marked SYNCING should not go into the ckpt; the new node
    will get those locks by receiving PLOCK_SYNC messages */
 
@@ -1659,6 +1665,9 @@ static int _unlink_checkpoint(struct lockspace *ls, SaNameT *name)
 	h = (SaCkptCheckpointHandleT) ls->plock_ckpt_handle;
 	log_group(ls, "unlink ckpt %llx", (unsigned long long)h);
 
+	if (!h)
+		return ret;
+
  unlink_retry:
 	rv = saCkptCheckpointUnlink(system_ckpt_handle, name);
 	if (rv == SA_AIS_ERR_TRY_AGAIN) {
@@ -1993,7 +2002,7 @@ void retrieve_plocks(struct lockspace *ls)
 			goto out_it;
 		}
 
-		if (!desc.sectionSize)
+		if (!desc.sectionId.idLen)
 			continue;
 
 		iov.sectionId = desc.sectionId;
@@ -2001,8 +2010,10 @@ void retrieve_plocks(struct lockspace *ls)
 		iov.dataSize = desc.sectionSize;
 		iov.dataOffset = 0;
 
+		/* for debug print */
 		memset(&buf, 0, sizeof(buf));
 		snprintf(buf, SECTION_NAME_LEN, "%s", desc.sectionId.id);
+
 		log_group(ls, "retrieve_plocks: section size %llu id %u \"%s\"",
 			  (unsigned long long)iov.dataSize, iov.sectionId.idLen,
 			  buf);
@@ -2020,13 +2031,14 @@ void retrieve_plocks(struct lockspace *ls)
 			goto out_it;
 		}
 
+		/* we'll get empty (zero length) sections for resources with
+		   no locks, which exist in ownership mode; the resource
+		   name and owner come from the section id */
+
 		log_group(ls, "retrieve_plocks: ckpt read %llu bytes",
 			  (unsigned long long)iov.readSize);
 		section_len = iov.readSize;
 
-		if (!section_len)
-		       continue;
-
 		if (section_len % sizeof(struct pack_plock)) {
 			log_error("retrieve_plocks: bad section len %d %s",
 				  section_len, ls->name);
@@ -2104,22 +2116,42 @@ int fill_plock_dump_buf(struct lockspace *ls)
 	struct posix_lock *po;
 	struct lock_waiter *w;
 	struct resource *r;
+	struct timeval now;
 	int rv = 0;
 	int len = DLMC_DUMP_SIZE, pos = 0, ret;
 
 	memset(plock_dump_buf, 0, sizeof(plock_dump_buf));
 	plock_dump_len = 0;
 
+	gettimeofday(&now, NULL);
+
 	list_for_each_entry(r, &ls->plock_resources, list) {
+
+		if (list_empty(&r->locks) &&
+		    list_empty(&r->waiters) &&
+		    list_empty(&r->pending)) {
+			ret = snprintf(plock_dump_buf + pos, len - pos,
+			      "%llu rown %d unused_ms %llu\n",
+			      (unsigned long long)r->number, r->owner,
+			      (unsigned long long)time_diff_ms(&r->last_access,
+				      			       &now));
+			if (ret >= len - pos) {
+				rv = -ENOSPC;
+				goto out;
+			}
+			pos += ret;
+			continue;
+		}
+
 		list_for_each_entry(po, &r->locks, list) {
 			ret = snprintf(plock_dump_buf + pos, len - pos,
-			      "%llu %s %llu-%llu nodeid %d pid %u owner %llx\n",
+			      "%llu %s %llu-%llu nodeid %d pid %u owner %llx rown %d\n",
 			      (unsigned long long)r->number,
 			      po->ex ? "WR" : "RD",
 			      (unsigned long long)po->start,
 			      (unsigned long long)po->end,
 			      po->nodeid, po->pid,
-			      (unsigned long long)po->owner);
+			      (unsigned long long)po->owner, r->owner);
 
 			if (ret >= len - pos) {
 				rv = -ENOSPC;
@@ -2130,13 +2162,30 @@ int fill_plock_dump_buf(struct lockspace *ls)
 
 		list_for_each_entry(w, &r->waiters, list) {
 			ret = snprintf(plock_dump_buf + pos, len - pos,
-			      "%llu WAITING %s %llu-%llu nodeid %d pid %u owner %llx\n",
+			      "%llu %s %llu-%llu nodeid %d pid %u owner %llx rown %d WAITING\n",
 			      (unsigned long long)r->number,
 			      w->info.ex ? "WR" : "RD",
 			      (unsigned long long)w->info.start,
 			      (unsigned long long)w->info.end,
 			      w->info.nodeid, w->info.pid,
-			      (unsigned long long)w->info.owner);
+			      (unsigned long long)w->info.owner, r->owner);
+
+			if (ret >= len - pos) {
+				rv = -ENOSPC;
+				goto out;
+			}
+			pos += ret;
+		}
+
+		list_for_each_entry(w, &r->pending, list) {
+			ret = snprintf(plock_dump_buf + pos, len - pos,
+			      "%llu %s %llu-%llu nodeid %d pid %u owner %llx rown %d PENDING\n",
+			      (unsigned long long)r->number,
+			      w->info.ex ? "WR" : "RD",
+			      (unsigned long long)w->info.start,
+			      (unsigned long long)w->info.end,
+			      w->info.nodeid, w->info.pid,
+			      (unsigned long long)w->info.owner, r->owner);
 
 			if (ret >= len - pos) {
 				rv = -ENOSPC;
